@@ -53,24 +53,24 @@ extern int blk_set_size (uint64_t new_size);
  * also ensure it is cached. */
 extern int blk_read (nbdkit_next *next,
                      uint64_t blknum, uint8_t *block, int *err)
-  __attribute__ ((__nonnull__ (1, 3, 4)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 4));
 
 /* As above, but read multiple blocks. */
 extern int blk_read_multiple (nbdkit_next *next,
                               uint64_t blknum, uint64_t nrblocks,
                               uint8_t *block, int *err)
-  __attribute__ ((__nonnull__ (1, 4, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 4, 5));
 
 /* If a single block is not cached, copy it from the plugin. */
 extern int blk_cache (nbdkit_next *next,
                       uint64_t blknum, uint8_t *block, int *err)
-  __attribute__ ((__nonnull__ (1, 3, 4)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 4));
 
 /* Write to the cache and the plugin. */
 extern int blk_writethrough (nbdkit_next *next,
                              uint64_t blknum, const uint8_t *block,
                              uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 3, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 5));
 
 /* Write a whole block.
  *
@@ -83,11 +83,11 @@ extern int blk_writethrough (nbdkit_next *next,
 extern int blk_write (nbdkit_next *next,
                       uint64_t blknum, const uint8_t *block,
                       uint32_t flags, int *err)
-  __attribute__ ((__nonnull__ (1, 3, 5)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 5));
 
 /* Iterates over each dirty block in the cache. */
 typedef int (*block_callback) (uint64_t blknum, void *vp);
 extern int for_each_dirty_block (block_callback f, void *vp)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 #endif /* NBDKIT_BLK_H */
