@@ -46,19 +46,19 @@ extern void blk_free (struct blk_overlay *blk);
 
 /* Allocate or resize the overlay and bitmap. */
 extern int blk_set_size (struct blk_overlay *blk, uint64_t new_size)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 /* Returns the status of the block in the overlay. */
 extern void blk_status (struct blk_overlay *blk,
                         uint64_t blknum, bool *present, bool *trimmed)
-  __attribute__ ((__nonnull__ (1)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1));
 
 /* Read a single block from the overlay or plugin. */
 extern int blk_read (struct blk_overlay *blk,
                      nbdkit_next *next,
                      uint64_t blknum, uint8_t *block,
                      bool cow_on_read, int *err)
-  __attribute__ ((__nonnull__ (1, 2, 4, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 4, 6));
 
 /* Read multiple blocks from the overlay or plugin. */
 extern int blk_read_multiple (struct blk_overlay *blk,
@@ -66,7 +66,7 @@ extern int blk_read_multiple (struct blk_overlay *blk,
                               uint64_t blknum, uint64_t nrblocks,
                               uint8_t *block,
                               bool cow_on_read, int *err)
-  __attribute__ ((__nonnull__ (1, 2, 5, 7)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 5, 7));
 
 /* Cache mode for blocks not already in overlay */
 enum cache_mode {
@@ -81,16 +81,16 @@ extern int blk_cache (struct blk_overlay *blk,
                       nbdkit_next *next,
                       uint64_t blknum, uint8_t *block, enum cache_mode,
                       int *err)
-  __attribute__ ((__nonnull__ (1, 2, 4, 6)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 2, 4, 6));
 
 /* Write a single block. */
 extern int blk_write (struct blk_overlay *blk,
                       uint64_t blknum, const uint8_t *block, int *err)
-  __attribute__ ((__nonnull__ (1, 3, 4)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3, 4));
 
 /* Trim a single block. */
 extern int blk_trim (struct blk_overlay *blk,
                      uint64_t blknum, int *err)
-  __attribute__ ((__nonnull__ (1, 3)));
+  NBDKIT_ATTRIBUTE_NONNULL ((1, 3));
 
 #endif /* NBDKIT_BLK_H */
