@@ -605,8 +605,10 @@ read_data_format (const char *value, struct allocator *a, uint64_t *size_rtn)
 
 static int parse_string (const char *value, size_t *start, size_t len,
                          string *rtn);
-static int parse_word (const char *value, size_t *start, size_t len,
-                       string *rtn);
+static int parse_le_be (string word, string *rtn);
+static int parse_base64 (string word, string *rtn);
+static int get_word (const char *value, size_t *start, size_t len,
+                     string *word);
 static size_t get_name (const char *value, size_t i, size_t len,
                         size_t *initial);
 static size_t get_var (const char *value, size_t i, size_t len,
@@ -918,11 +920,27 @@ parser (int level, const char *value, size_t *start, size_t len,
       APPEND_EXPR (new_node (expr (EXPR_BYTE, j)));
       break;
 
-    case 'l': case 'b': {       /* le or be + NN: + WORD */
+    case 'l': case 'b': {       /* le or be + NN: + WORD, or base64:... */
+      CLEANUP_FREE_STRING string word = empty_vector;
       string str = empty_vector;
 
-      if (parse_word (value, &i, len, &str) == -1)
+      /* Get the whole word, up to the next whitespace (or end of
+       * input), into a \0-terminated string.
+       */
+      if (get_word (value, &i, len, &word) == -1)
         return -1;
+      /* This must be true because we parsed 'l' or 'b' above, plus
+       * the terminating \0 character makes at least 2.
+       */
+      assert (word.len >= 2);
+
+      if (strncmp (word.ptr, "base64:", 7) == 0) {
+        if (parse_base64 (word, &str) == -1)
+          return -1;
+      } else {
+        if (parse_le_be (word, &str) == -1)
+          return -1;
+      }
       APPEND_EXPR (new_node (expr (EXPR_STRING, str)));
       break;
     }
@@ -955,6 +973,36 @@ parser (int level, const char *value, size_t *start, size_t len,
   return 0;
 }
 
+/* Return the next word in the input, starting at the current position
+ * and ending at the first whitespace (or end of input).  This updates
+ * *start with the end of the word + 1.
+ */
+static int
+get_word (const char *value, size_t *start, size_t len, string *word)
+{
+  size_t i = *start;
+  size_t n;
+
+  while (i < len) {
+    if (ascii_isspace (value[i]))
+      break;
+    i++;
+  }
+  n = i - *start;
+
+  *word = (string) empty_vector;
+  if (string_reserve (word, n + 1) == -1) {
+    nbdkit_error ("realloc: %m");
+    return -1;
+  }
+  memcpy (word->ptr, &value[*start], n);
+  word->ptr[n] = '\0';
+  word->len = n + 1;
+  *start = i;
+
+  return 0;
+}
+
 /* Return the next \NAME in the input.  This skips whitespace, setting
  * *initial to the index of the start of the NAME (minus backslash).
  * It returns the length of NAME (minus backslash), or 0 if not found.
@@ -1110,11 +1158,8 @@ parse_string (const char *value, size_t *start, size_t len, string *rtn)
 
 /* Parse le<NN>:WORD be<NN>:WORD expressions.  These are parsed into strings. */
 static int
-parse_word (const char *value, size_t *start, size_t len, string *rtn)
+parse_le_be (const string word, string *rtn)
 {
-  size_t i = *start;
-  CLEANUP_FREE_STRING string copy = empty_vector;
-  size_t n;
   enum { little, big } endian;
   uint16_t u16;
   uint32_t u32;
@@ -1122,27 +1167,6 @@ parse_word (const char *value, size_t *start, size_t len, string *rtn)
 
   *rtn = (string) empty_vector;
 
-  /* It's convenient to use the nbdkit_parse* functions below because
-   * they deal already with overflow etc.  However these functions
-   * require a \0-terminated string, so we have to copy what we are
-   * parsing to a new string here.
-   */
-  while (i < len) {
-    if (ascii_isspace (value[i]))
-      break;
-    i++;
-  }
-  n = i - *start;
-  assert (n > 0);          /* must be because caller parsed 'l'/'b' */
-  if (string_reserve (&copy, n + 1) == -1) {
-    nbdkit_error ("realloc: %m");
-    return -1;
-  }
-  memcpy (copy.ptr, &value[*start], n);
-  copy.ptr[n] = '\0';
-  copy.len = n + 1;
-  *start = i;
-
   /* Reserve enough space in the return buffer for the longest
    * possible bitstring (64 bits / 8 bytes).
    */
@@ -1152,42 +1176,42 @@ parse_word (const char *value, size_t *start, size_t len, string *rtn)
   }
 
   /* Parse the rest of {le|be}{16|32|64}: */
-  if (strncmp (copy.ptr, "le16:", 5) == 0) {
+  if (strncmp (word.ptr, "le16:", 5) == 0) {
     endian = little; rtn->len = 2;
   }
-  else if (strncmp (copy.ptr, "le32:", 5) == 0) {
+  else if (strncmp (word.ptr, "le32:", 5) == 0) {
     endian = little; rtn->len = 4;
   }
-  else if (strncmp (copy.ptr, "le64:", 5) == 0) {
+  else if (strncmp (word.ptr, "le64:", 5) == 0) {
     endian = little; rtn->len = 8;
   }
-  else if (strncmp (copy.ptr, "be16:", 5) == 0) {
+  else if (strncmp (word.ptr, "be16:", 5) == 0) {
     endian = big; rtn->len = 2;
   }
-  else if (strncmp (copy.ptr, "be32:", 5) == 0) {
+  else if (strncmp (word.ptr, "be32:", 5) == 0) {
     endian = big; rtn->len = 4;
   }
-  else if (strncmp (copy.ptr, "be64:", 5) == 0) {
+  else if (strncmp (word.ptr, "be64:", 5) == 0) {
     endian = big; rtn->len = 8;
   }
   else {
     nbdkit_error ("data parameter: expected \"le16/32/64:\" "
-                  "or \"be16/32/64:\" at offset %zu", i);
+                  "or \"be16/32/64:\" but got \"%s\"", word.ptr);
     return -1;
   }
 
   /* Parse the word field into a host-order unsigned int. */
   switch (rtn->len) {
   case 2:
-    if (nbdkit_parse_uint16_t ("data", &copy.ptr[5], &u16) == -1)
+    if (nbdkit_parse_uint16_t ("data", &word.ptr[5], &u16) == -1)
       return -1;
     break;
   case 4:
-    if (nbdkit_parse_uint32_t ("data", &copy.ptr[5], &u32) == -1)
+    if (nbdkit_parse_uint32_t ("data", &word.ptr[5], &u32) == -1)
       return -1;
     break;
   case 8:
-    if (nbdkit_parse_uint64_t ("data", &copy.ptr[5], &u64) == -1)
+    if (nbdkit_parse_uint64_t ("data", &word.ptr[5], &u64) == -1)
       return -1;
     break;
   default: abort ();
@@ -1230,6 +1254,43 @@ parse_word (const char *value, size_t *start, size_t len, string *rtn)
   return 0;
 }
 
+/* Parse base64:... sequence. */
+static int
+parse_base64 (const string word, string *rtn)
+{
+#ifdef NBDKIT_DATA_HAVE_BASE64_SUPPORT
+  gnutls_datum_t in, out;
+  int err;
+
+  assert (strncmp (word.ptr, "base64:", 7) == 0); /* caller checked already */
+
+  in.data = (unsigned char *) &word.ptr[7];
+  in.size = word.len - 7 - 1;
+  err = gnutls_base64_decode2 (&in, &out);
+  if (err != GNUTLS_E_SUCCESS) {
+    nbdkit_error ("base64: %s", gnutls_strerror (err));
+    return -1;
+  }
+
+  /* Reserve space in the return string. */
+  *rtn = (string) empty_vector;
+  if (string_reserve_exactly (rtn, out.size) == -1) {
+    nbdkit_error ("realloc: %m");
+    return -1;
+  }
+
+  memcpy (rtn->ptr, out.data, out.size);
+  free (out.data);
+  rtn->len = out.size;
+
+  return 0;
+
+#else
+  nbdkit_error ("base64 is not supported in this build of the plugin");
+  return -1;
+#endif
+}
+
 /* This simple optimization pass over the AST simplifies some
  * expressions.
  */
