@@ -148,7 +148,7 @@ create_filesystem (struct virtual_disk *disk)
 static int64_t
 estimate_size (void)
 {
-  CLEANUP_FREE char *command = NULL, *line = NULL, *lastline = NULL;
+  CLEANUP_FREE char *command = NULL, *line = NULL;
   size_t len = 0;
   FILE *fp;
   int64_t ret;
@@ -162,6 +162,7 @@ estimate_size (void)
   }
   fprintf (fp, "du -c -k -s ");
   shell_quote (dir, fp);
+  fprintf (fp, " | tail -n1");
   if (fclose (fp) == EOF) {
     nbdkit_error ("memstream failed: %m");
     return -1;
@@ -175,20 +176,13 @@ estimate_size (void)
     return -1;
   }
 
-  /* Ignore everything up to the last line. */
-  len = 0;
-  while (getline (&line, &len, fp) != -1) {
-    nbdkit_debug ("du: %s", line);
-    free (lastline);
-    lastline = strndup (line, len);
-    if (lastline == NULL) {
-      nbdkit_error ("strndup: %m");
-      pclose (fp);
-      return -1;
-    }
-  }
-  if (ferror (fp)) {
-    nbdkit_error ("getline failed: %m");
+  /* Should only be one line of input. */
+  errno = 0;
+  if (getline (&line, &len, fp) == -1 || ferror (fp)) {
+    if (errno)
+      nbdkit_error ("getline failed: %m");
+    else
+      nbdkit_error ("du: no output");
     pclose (fp);
     return -1;
   }
@@ -201,14 +195,9 @@ estimate_size (void)
   if (exit_status_to_nbd_error (r, "pclose: du") == -1)
     return -1;
 
-  if (lastline == NULL) {
-    nbdkit_error ("no output from du command");
-    return -1;
-  }
-
   /* Parse the last line. */
-  if (sscanf (lastline, "%" SCNi64, &ret) != 1 || ret < 0) {
-    nbdkit_error ("could not parse last line from du command: %s", lastline);
+  if (sscanf (line, "%" SCNi64, &ret) != 1 || ret < 0) {
+    nbdkit_error ("could not parse last line from du command: %s", line);
     return -1;
   }
 
