@@ -41,6 +41,7 @@
 
 #define CL_TARGET_OPENCL_VERSION 200 /* OpenCL >= 2.0 */
 #include <CL/cl.h>
+#include <CL/cl_ext.h>
 
 #define NBDKIT_API_VERSION 2
 #include <nbdkit-plugin.h>
@@ -126,6 +127,12 @@ get_all_devices (void)
   /* Build the list of all devices from all platforms as a flat list. */
   what = "clGetPlatformIDs";
   r = clGetPlatformIDs (0, NULL, &num_platforms);
+  if (r == CL_PLATFORM_NOT_FOUND_KHR) {
+    /* OpenCL seems to return this when no platform is detected at
+     * all, so just return the empty list in this case.
+     */
+    return;
+  }
   if (r != CL_SUCCESS) goto err;
   platform_ids = calloc (num_platforms, sizeof platform_ids[0]);
   if (!platform_ids) {
@@ -140,6 +147,13 @@ get_all_devices (void)
     what = "clGetDeviceIDs";
     r = clGetDeviceIDs (platform_ids[pl_i], CL_DEVICE_TYPE_ALL, 0, NULL,
                         &num_devices);
+    if (r == CL_DEVICE_NOT_FOUND) {
+      /* OpenCL seems to return this when the platform is found but
+       * the vendor's library is not configured to access devices
+       * within the platform; skip over this platform.
+       */
+      continue;
+    }
     if (r != CL_SUCCESS) goto err;
     free (device_ids);
     device_ids = calloc (num_devices, sizeof device_ids[0]);
@@ -797,8 +811,15 @@ vram_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   /* Aligned body */
   while (count >= BUFFER_SIZE) {
     free_buffer (bufnum);
+
+    count -= BUFFER_SIZE;
+    offset += BUFFER_SIZE;
+    bufnum++;
   }
 
+  if (flags & NBDKIT_FLAG_FUA && vram_flush (handle, 0) == -1)
+    return -1;
+
   return 0;
 }
 
