@@ -45,7 +45,7 @@ cleanup_fn rm -f $out
 #
 # Note here we have to run the server directly because the wrapper
 # will rewrite the plugin name.
-../server/nbdkit nosuchname 2>$out ||:
+../server/nbdkit$EXEEXT nosuchname 2>$out ||:
 cat $out
 grep 'nosuchname' $out
 # The error message should suggest a Fedora or Debian package name:
